@@ -48,6 +48,8 @@ class Verified(Disposition):
         * ``location``: The user's stated location
         * ``name``: The user's display/familiar name
         * ``pronouns``: The user's declared pronouns
+        * ``endpoints``: A dictionary of key-value pairs referring to the user's
+          various integration services
 
     """
 
