@@ -216,9 +216,7 @@ class AuthlFlask:
         browser window closing
 
     :param tokens.TokenStore token_storage: Storage for token data for
-        methods which use it. Defaults to :py:class:`tokens.Serializer` using
-        the Flask ``app.secret_key``; this is suitable for load-balancing
-        scenarios as long as all service nodes use the same secret_key.
+        methods which use it. Uses the same default as :py:func:`authl.from_config`.
 
         Note that if the default is used, the ``app.secret_key`` **MUST** be set
         before this class is initialized.
@@ -249,18 +247,14 @@ class AuthlFlask:
                  stylesheet: typing.Union[str, typing.Callable] = None,
                  on_verified: typing.Callable = None,
                  make_permanent: bool = True,
-                 state_storage: dict = None,
+                 state_storage: typing.Dict = None,
                  token_storage: tokens.TokenStore = None,
                  session_namespace='_authl',
                  ):
         # pylint:disable=too-many-arguments,too-many-locals,too-many-statements
 
         if state_storage is None:
-            state_storage = flask.session
-
-        if token_storage is None:
-            assert app.secret_key, "app.secret_key must be set to use default token_storage"
-            token_storage = tokens.Serializer(app.secret_key)
+            state_storage = typing.cast(typing.Dict, flask.session)
 
         self.authl = from_config(
             config,
@@ -322,7 +316,7 @@ class AuthlFlask:
 
             LOGGER.info("Successful login: %s", disp.identity)
             if self._session_auth_name is not None:
-                flask.session.permanent = self.make_permanent
+                flask.session.permanent = self.make_permanent  # pylint:disable=assigning-non-slot
                 flask.session[self._session_auth_name] = disp.identity
 
             if self._on_verified:
