@@ -18,6 +18,7 @@ whatever framework you're using. Please note also that the scheme (``http`` vs.
 """
 
 import logging
+import secrets
 import time
 import typing
 import urllib.parse
@@ -34,10 +35,10 @@ LOGGER = logging.getLogger(__name__)
 
 # We do this instead of functools.lru_cache so that IndieAuth.handles_page
 # and find_endpoint can both benefit from the same endpoint cache
-_ENDPOINT_CACHE = expiringdict.ExpiringDict(max_len=128, max_age_seconds=1800)
+_ENDPOINT_CACHE = expiringdict.ExpiringDict(max_len=128, max_age_seconds=300)
 
-# And similar for retrieving user profiles
-_PROFILE_CACHE = expiringdict.ExpiringDict(max_len=128, max_age_seconds=1800)
+# And similar for retrieving user h-cards
+_PROFILE_CACHE = expiringdict.ExpiringDict(max_len=128, max_age_seconds=300)
 
 
 def find_endpoint(id_url: str,
@@ -73,7 +74,6 @@ def find_endpoints(id_url: str,
     profile = id_url
 
     def _derive_endpoint(links, content, rel) -> typing.Optional[str]:
-        LOGGER.debug('links for %s: %s', id_url, links)
         if links and rel in links:
             LOGGER.debug("%s: Found %s link header: %s", id_url, rel, links[rel]['url'])
             return links[rel]['url']
@@ -103,6 +103,8 @@ def find_endpoints(id_url: str,
 
     # Derive the useful IndieAuth endpoints
     endpoints = cached.copy()
+
+    LOGGER.debug('links for %s: %s', id_url, links)
     for rel in (
         'authorization_endpoint',
         'token_endpoint',
@@ -160,34 +162,62 @@ def _parse_hcard(id_url, card):
     }
 
 
-def get_profile(id_url: str, links=None, content: BeautifulSoup = None, endpoints=None) -> dict:
-    """ Given an identity URL, try to parse out an Authl profile """
-    if not content:
-        if id_url in _PROFILE_CACHE:
-            LOGGER.debug("Reusing %s profile from cache", id_url)
-            return _PROFILE_CACHE[id_url]
+def get_profile(id_url: str,
+                server_profile: dict = None,
+                links=None,
+                content: BeautifulSoup = None,
+                endpoints=None) -> dict:
+    """ Given an identity URL, try to parse out an Authl profile
 
+    :param str id_url: The profile page to parse
+    :param dict server_profile: An IndieAuth response profile
+    :param dict links: Profile response's links dictionary
+    :param content: Pre-parsed page content
+    :param dict endpoints: Pre-parsed page endpoints
+    """
+
+    if id_url in _PROFILE_CACHE:
+        LOGGER.debug("Reusing %s profile from cache", id_url)
+        profile = _PROFILE_CACHE[id_url].copy()
+    else:
+        profile = {}
+
+    if not content and id_url not in _PROFILE_CACHE:
         LOGGER.debug("get_profile: Retrieving %s", id_url)
         request = utils.request_url(id_url)
         if request is not None:
             links = request.links
             content = BeautifulSoup(request.text, 'html.parser')
 
-    h_cards = mf2py.Parser(doc=content).to_dict(filter_by_type="h-card")
-    LOGGER.debug("get_profile(%s): found %d h-cards", id_url, len(h_cards))
+    if content:
+        profile = {}
+        h_cards = mf2py.Parser(doc=content).to_dict(filter_by_type="h-card")
+        LOGGER.debug("get_profile(%s): found %d h-cards", id_url, len(h_cards))
 
-    profile = {}
-    for card in h_cards:
-        items = _parse_hcard(id_url, card)
+        for card in h_cards:
+            items = _parse_hcard(id_url, card)
 
-        profile.update({k: v for k, v in items.items() if v and k not in profile})
+            profile.update({k: v for k, v in items.items() if v and k not in profile})
+
+    # Only stash the version without the IndieAuth server profile addons, in case
+    # the user logs in again without the profile/email scopes
+    LOGGER.debug("Stashing %s profile", id_url)
+    _PROFILE_CACHE[id_url] = profile.copy()
+
+    if server_profile:
+        # The IndieAuth server also provided a profile, which should supercede the h-card
+        for in_key, out_key in (('name', 'name'),
+                                ('photo', 'avatar'),
+                                ('url', 'homepage'),
+                                ('email', 'email')):
+            if in_key in server_profile:
+                profile[out_key] = server_profile[in_key]
 
     if not endpoints:
         endpoints, _ = find_endpoints(id_url, links=links, content=content)
     if endpoints:
         profile['endpoints'] = endpoints
 
-    _PROFILE_CACHE[id_url] = profile
     return profile
 
 
@@ -225,15 +255,7 @@ def verify_id(request_id: str, response_id: str) -> str:
 
 
 class IndieAuth(Handler):
-    """ Supports login via IndieAuth.
-
-    **SECURITY NOTE:** When used with tokens.Serializer, this is subject to certain
-    classes of replay attack; for example, if the user endpoint uses irrevocable
-    signed tokens for the code grant (which is done in many endpoints, e.g.
-    SelfAuth), an attacker can replay a transaction that it intercepts. As such
-    it is very important that the timeout be configured to a reasonably short
-    time to mitigate this.
-    """
+    """ Supports login via IndieAuth. """
 
     @property
     def service_name(self):
@@ -263,7 +285,12 @@ class IndieAuth(Handler):
         :param client_id: The client_id to send to the remote IndieAuth
             provider. Can be a string or a function that returns a string.
 
-        :param token_store: Storage for the tokens
+        :param token_store: Storage for the tokens.
+
+            ***Security note:*** :py:class:`tokens.Serializer` is not supported,
+            as it makes this handler subject to replay attacks when used with
+            many common IndieAuth servers, and also prevents PKCE from being
+            effective.
 
         :param int timeout: Maximum time to wait for login to complete
             (default: 600)
@@ -274,6 +301,11 @@ class IndieAuth(Handler):
         self._token_store = token_store
         self._timeout = timeout or 600
 
+        if isinstance(token_store, tokens.Serializer):
+            LOGGER.error(
+                "Cannot use tokens.Serializer with IndieAuth due to security considerations")
+            raise ValueError("Cannot use tokens.Serializer with IndieAuth")
+
     def handles_url(self, url):
         """
         If this page is already known to have an IndieAuth endpoint, we reuse
@@ -293,7 +325,9 @@ class IndieAuth(Handler):
         if not endpoint:
             return disposition.Error("Failed to get IndieAuth endpoint", redir)
 
-        state = self._token_store.put((id_url, endpoint, callback_uri, time.time(), redir))
+        verifier = secrets.token_urlsafe()
+        state = self._token_store.put(
+            (id_url, endpoint, callback_uri, verifier, time.time(), redir))
 
         client_id = utils.resolve_value(self._client_id)
         LOGGER.debug("Using client_id %s", client_id)
@@ -302,20 +336,22 @@ class IndieAuth(Handler):
             'redirect_uri': callback_uri,
             'client_id': client_id,
             'state': state,
+            'code_challenge': utils.pkce_challenge(verifier),
+            'code_challenge_method': 'S256',
             'response_type': 'code',
-            'scope': 'profile',
+            'scope': 'profile email',
             'me': id_url})
         return disposition.Redirect(url)
 
     def check_callback(self, url, get, data):
-        # pylint:disable=too-many-return-statements
+        # pylint:disable=too-many-return-statements,too-many-locals
 
         state = get.get('state')
         if not state:
             return disposition.Error("No transaction provided", '')
 
         try:
-            id_url, endpoint, callback_uri, when, redir = self._token_store.pop(state)
+            id_url, endpoint, callback_uri, verifier, when, redir = self._token_store.pop(state)
         except (KeyError, ValueError):
             return disposition.Error("Invalid token", '')
 
@@ -327,7 +363,8 @@ class IndieAuth(Handler):
             request = requests.post(endpoint, data={
                 'code': get['code'],
                 'client_id': utils.resolve_value(self._client_id),
-                'redirect_uri': callback_uri
+                'redirect_uri': callback_uri,
+                'code_verifier': verifier,
             }, headers={'accept': 'application/json'})
 
             if request.status_code != 200:
@@ -345,7 +382,10 @@ class IndieAuth(Handler):
                 return disposition.Error("Got invalid response JSON", redir)
 
             response_id = verify_id(id_url, response['me'])
-            return disposition.Verified(response_id, redir, get_profile(response_id))
+            return disposition.Verified(
+                response_id, redir,
+                get_profile(response_id,
+                            server_profile=response.get('profile')))
         except KeyError as key:
             return disposition.Error("Missing " + str(key), redir)
         except ValueError as err:
